import { HTMLAttributes, useState } from "react";
import { z } from "zod";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { cn } from "@/lib/utils";
import { Button } from "@/components/ui/button";
import {
	Form,
	FormControl,
	FormField,
	FormItem,
	FormLabel,
	FormMessage
} from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { PasswordInput } from "@/components/password-input";
import { registerUser } from "@/lib/api";
import { ApiError } from "@/errors/api-error";

type SignUpFormProps = HTMLAttributes<HTMLDivElement>;

const formSchema = z
	.object({
		username: z
			.string()
			.min(1, { message: "Please enter your username" })
			.min(3, { message: "Username must be at least 3 characters long" })
			.max(64, { message: "Username must be less than 64 characters" }),
		email: z
			.string()
			.min(1, { message: "Please enter your email" })
			.email({ message: "Invalid email address" }),
		phone_number: z
			.string()
			.min(1, { message: "Please enter your phone number" })
			.min(2, { message: "Phone number must be at least 2 digits" })
			.max(15, { message: "Phone number cannot be longer than 15 digits" })
			.regex(/^\+[1-9]\d{1,14}$/, { message: "Invalid phone number format" }),
		password: z
			.string()
			.min(1, { message: "Please enter your password" })
			.min(8, {
				message: "Password must be at least 8 characters long"
			})
			.max(128, { message: "Password must be at most 128 characters long" })
			.refine((password) => /[A-Z]/.test(password), {
				message: "Password must contain at least one uppercase letter"
			})
			.refine((password) => /[a-z]/.test(password), {
				message: "Password must contain at least one lowercase letter"
			})
			.refine((password) => /\d/.test(password), {
				message: "Password must contain at least one number"
			})
			.refine((password) => /[@$!%*?&]/.test(password), {
				message:
					"Password must contain at least one special character (@, $, !, %, *, ?, &)"
			}),
		confirmPassword: z.string()
	})
	.refine((data) => data.password === data.confirmPassword, {
		message: "Passwords don't match.",
		path: ["confirmPassword"]
	});

export function SignUpForm({ className, ...props }: SignUpFormProps) {
	const [isLoading, setIsLoading] = useState(false);
	const [error, setError] = useState<string | null>(null);
	const [successMessage, setSuccessMessage] = useState<string | null>(null);

	const form = useForm<z.infer<typeof formSchema>>({
		resolver: zodResolver(formSchema),
		defaultValues: {
			username: "",
			email: "",
			phone_number: "",
			password: "",
			confirmPassword: ""
		}
	});

	async function onSubmit(data: z.infer<typeof formSchema>) {
		setIsLoading(true);
		setError(null);
		setSuccessMessage(null);

		try {
			await registerUser({
				username: data.username,
				email: data.email,
				password: data.password,
				phone_number: data.phone_number
			});
			setSuccessMessage("Account created successfully!");
		} catch (error: unknown) {
			setError(error instanceof ApiError ? error.response : "Something went wrong.");
		} finally {
			setIsLoading(false);
		}
	}

	return (
		<div className={cn("grid gap-6", className)} {...props}>
			<Form {...form}>
				<form onSubmit={form.handleSubmit(onSubmit)}>
					<div className="grid gap-2">
						<FormField
							control={form.control}
							name="username"
							render={({ field }) => (
								<FormItem className="space-y-1">
									<FormLabel>Username</FormLabel>
									<FormControl>
										<Input placeholder="your_username" {...field} />
									</FormControl>
									<FormMessage />
								</FormItem>
							)}
						/>
						<FormField
							control={form.control}
							name="email"
							render={({ field }) => (
								<FormItem className="space-y-1">
									<FormLabel>Email</FormLabel>
									<FormControl>
										<Input placeholder="name@example.com" {...field} />
									</FormControl>
									<FormMessage />
								</FormItem>
							)}
						/>
						<FormField
							control={form.control}
							name="phone_number"
							render={({ field }) => (
								<FormItem className="space-y-1">
									<FormLabel>Phone Number</FormLabel>
									<FormControl>
										<Input placeholder="+1234567890" {...field} />
									</FormControl>
									<FormMessage />
								</FormItem>
							)}
						/>
						<FormField
							control={form.control}
							name="password"
							render={({ field }) => (
								<FormItem className="space-y-1">
									<FormLabel>Password</FormLabel>
									<FormControl>
										<PasswordInput placeholder="********" {...field} />
									</FormControl>
									<FormMessage />
								</FormItem>
							)}
						/>
						<FormField
							control={form.control}
							name="confirmPassword"
							render={({ field }) => (
								<FormItem className="space-y-1">
									<FormLabel>Confirm Password</FormLabel>
									<FormControl>
										<PasswordInput placeholder="********" {...field} />
									</FormControl>
									<FormMessage />
								</FormItem>
							)}
						/>
						{error && <p className="text-red-500">{error}</p>}
						{successMessage && (
							<p className="text-green-500">{successMessage}</p>
						)}
						<Button className="mt-2" type="submit" disabled={isLoading}>
							{isLoading ? "Creating Account..." : "Create Account"}
						</Button>
					</div>
				</form>
			</Form>
		</div>
	);
}
